import six


class Region(object):
    def __init__(self, id=None):
        self.id = id
        if self.id is None or not isinstance(self.id, six.string_types):
            raise ValueError('id must be a string and have a value')

    def serialize(self):
        return {'id': self.id}



import httplib


def set_task_percentage(context, task_id, percentage):
    if task_id is None:
        return

    data = {'percentage': percentage}
    response = context.api.post(path='/tasks/%s/update' % task_id, data=data)
    if response.status_code != httplib.OK:
        raise Exception(
            'Got bad response from tasks API: %d, %s' % (
                response.status_code, response.content))



from . import Resource


class Appliance(Resource):
    def __init__(self, size_b=None, type_id=None, **kwargs):
        super(Appliance, self).__init__(type='appliance', **kwargs)
        self.type_id = type_id

    def serialize(self):
        data = super(Appliance, self).serialize()
        data['details'] = {
            self.type: {
                "type_id": self.type_id
            }
        }
        return data



from . import Resource


class Server(Resource):
    def __init__(self, cpu_hz=None, cpu_cores=None, ram_b=None, volumes_b=None,
                 state=None, provider_state=None, image_detail=None,
                 instance_type=None, ip_addresses=None,
                 is_virtual=None, **kwargs):
        super(Server, self).__init__(type='server', **kwargs)
        self.cpu_cores = cpu_cores
        self.cpu_hz = cpu_hz
        self.ram_b = ram_b
        self.volumes_b = volumes_b
        self.is_virtual = is_virtual

        if state is None:
            state = 'unknown'
        self.state = state

        if provider_state is None:
            provider_state = 'unknown'
        self.provider_state = provider_state

        self.instance_type = instance_type
        self.image_detail = image_detail

        if ip_addresses is None:
            ip_addresses = []
        self.ip_addresses = ip_addresses

    def serialize(self):
        data = super(Server, self).serialize()

        ip_addresses = [ip.serialize() for ip in self.ip_addresses]

        data['details'] = {
            self.type: {
                "cpu_cores": self.cpu_cores,
                "cpu_hz": self.cpu_hz,
                "ram_b": self.ram_b,
                "volumes_b": self.volumes_b,
                "instance_type": self.instance_type,
                "state": self.state,
                "provider_state": self.provider_state,
                "ip_addresses": ip_addresses,
            }
        }

        type_details = data['details'][self.type]

        if self.image_detail is not None:
            type_details['image_detail'] = self.image_detail.serialize()
        else:
            type_details['image_detail'] = None

        if self.is_virtual is not None:
            type_details['is_virtual'] = self.is_virtual

        return data



class IpAddress(object):
    def __init__(self, ip_address=None, network_id=None, network_name=None):
        self.ip_address = ip_address
        self.network_id = network_id
        self.network_name = network_name

    def serialize(self):
        results = {}
        if self.ip_address is not None:
            results['ip_address'] = self.ip_address

        if self.network_id is not None and self.network_name is not None:
            results['network'] = {
                'id': self.network_id,
                'name': self.network_name,
            }

        return results



from dateutil import parser
import pytz
import six


def convert_datetime(dt):
    if dt is None or dt == '':
        return None
    if isinstance(dt, six.string_types):
        dt = parser.parse(dt)
    if dt.tzinfo:
        dt = dt.astimezone(pytz.utc).replace(tzinfo=None)
    if dt.microsecond > 0:
        dt = dt.strftime('%Y-%m-%dT%H:%M:%S.%fZ')
    else:
        dt = dt.strftime('%Y-%m-%dT%H:%M:%SZ')
    return dt



import six


class ImageDetail(object):
    def __init__(self, id=None, name=None, type=None, distribution=None,
                 version=None, architecture=None):
        self.id = id
        self.name = name
        self.type = type
        self.distribution = distribution
        self.version = version
        self.architecture = architecture

        self._check_not_none_str_value('id', self.id)

    def _check_not_none_str_value(self, name, value):
        if value is None or not isinstance(value, six.string_types):
            raise ValueError('%s must be a string and have a value' % name)

    def serialize(self):
        return {
            "id": self.id,
            "name": self.name,
            "architecture": self.architecture,
            "distribution": self.distribution,
            "type": self.type,
            "version": self.version,
        }


class ImageDetailMap(object):
    def __init__(self, mapping=None):
        self.mapping = mapping

    def get(self, id=None, name=None, version=None, type=None,
            architecture=None):
        mapped_image = self.mapping.get(id)
        if mapped_image is None:
            (mname, mtype, mdist, mversion, march) = tuple([None] * 5)
        else:
            (mname, mtype, mdist, mversion, march) = mapped_image

        mname = mname or name
        mtype = mtype or type
        mversion = mversion or version
        march = march or architecture

        return ImageDetail(
            id=id,
            name=mname,
            type=mtype,
            distribution=mdist,
            version=mversion,
            architecture=march,
        )



class Metadata(object):
    DEFAULT_NAMESPACE_ID = 'default'
    PROVIDER_SPECIFIC_NAMESPACE_ID = 'provider_specific'
    PROVIDER_SPECIFIC_EDITABLE_NAMESPACE_ID = 'provider_specific_editable'
    PROVIDER_METADATA_NAMESPACE_ID = 'provider_metadata'

    def __init__(self, values=None, default_namespace=None):
        if default_namespace is None:
            default_namespace = self.PROVIDER_SPECIFIC_NAMESPACE_ID
        self.default_namespace = default_namespace
        self.data = {}

        if values is not None:
            for value in values:
                if len(value) == 2:
                    self.add(value[0], value[1])
                else:
                    self.add(value[1], value[2], namespace=value[0])

    def add(self, key, value, namespace=None):
        if namespace is None:
            namespace = self.default_namespace
        if namespace not in self.data:
            self.data[namespace] = {}
        self.data[namespace][key] = value
        return self

    def serialize(self):
        return self.data
